var http = require('http');
var fs = require('fs');
var SerialPort = require('serialport');
var url = require('url');
const port = 8080;

var serial;
function createComPort(comPort, callback) {
	serial = new SerialPort(comPort, function(err) {
		if (err) {
			console.log("createComPort error: " + err);
			callback("error: " + err);
		} else {
			callback(null);

		}
	});
}

function writeToComPort(dataToWrite, callback) {
	serial.write(dataToWrite, function(err) {
		if (err) {
			console.log("writeToComPort error: " + err);
			callback("error: " + err);
		} else {
			callback(null);
		}
	})
}

function closeSerialPort(callback) {
	serial.close(function(err) {
		if (err) {
			console.log("closeSerialPort error: " + err);
			callback(err);
		} else {
			callback(null);
		}
	});
}

function getData(callback) {
	serial.on('data', function(data) {
		closeSerialPort(function(err) {
			if (err !== null) {
				console.log("getData error: " + err);
				callback('error');
			} else
				callback(data);
		});
	});
}

function comPortRequest(comPort, cmd, callback) {
	createComPort(comPort, function(err) {
		if (err != null) {
			callback(err);
		} else {
			writeToComPort(cmd, function(err) {
				if (err != null) {
					console.log('comPortRequest error: ' + err);
					callback("Error");
				} else {
					getData(function(respond) {
						callback(respond);
					})
				}
			})
		}
	})
}

function getComPorts(callback) {
	SerialPort.list(function(err, ports) {
		var i = 0;
		var list = [ 'dsa' ];
		ports.forEach(function(port) {
			list[i] = port.comName;
			i++;
		});
		callback(err, list);
	})
	// SerialPort.list();
}

http.createServer(function(serverRequest, response) {
	console.log('request starting...');

	if (serverRequest.url == "/") {
		var filePath = './index.html';
		var contentType = 'text/html';

		fs.readFile(filePath, function(error, content) {
			// console.log(filePath);
			if (error) {
				if (error.code == 'ENOENT') {
					fs.readFile('./404.html', function(error, content) {
						response.writeHead(200, {
							'Content-Type' : contentType
						});
						response.end(content, 'utf-8');
					});
				} else {
					response.writeHead(500);
					response.end('Sorry, check with the site admin for error: ' + error.code + ' ..\n');
					response.end();
				}
			} else {
				response.writeHead(200, {
					'Content-Type' : contentType
				});
				response.end(content, 'utf-8');
			}
		})
	} else if (serverRequest.url.startsWith("/getComPorts")) {
		getComPorts(function(error, list) {
			if (error != null) {
				console.log("error");
				response.writeHead(200, {
					'Content-Type' : 'text/plain'
				});
				response.write("error: " + error);
			} else {
				response.writeHead(200, {
					'Content-Type' : 'application/json'
				});
				var toSend = createJsonRespondWithComPorts(list.length, list);
				response.write(toSend);
			}
			response.end();
		})
	} else if (serverRequest.url.startsWith("/getRelays") || serverRequest.url.startsWith("/setRelays")) {
		var queryData = url.parse(serverRequest.url, true).query;
		comPortRequest(queryData.comPort, queryData.cmd, function(data) {
			if (data.toString().startsWith('Error')) {
				response.writeHead(200, {
					'Content-Type' : 'text/plain'
				});
				response.end('Error');
			} else {
				comPortRequest(queryData.comPort, 'ask//', function(newData) {
					if (data.toString().startsWith('Error')) {
						response.writeHead(200, {
							'Content-Type' : 'text/plain'
						});
						response.end('Error');
					} else {
						response.writeHead(200, {
							'Content-Type' : 'text/plain'
						});
						response.end(createJsonRespondWithStates(parseInt(newData.toString('hex'), 16)));
						console.log('response.end');
					}
				});
			}
		});
	} else if (serverRequest.url.startsWith("/reset")) {
		serial.close(function(err){
			if(err)
				console.log('not closed');
			else 
				console.log('closed')
		});
		response.writeHead(200, {
			'Content-Type' : 'text/plain'
		});
		response.end('Error');
		console.log('reset');
	} else {
		var filePath = './404.html';
		var contentType = 'text/html';

		fs.readFile(filePath, function(error, content) {
			if (error) {
				if (error.code == 'ENOENT') {
					fs.readFile('./404.html', function(error, content) {
						response.writeHead(200, {
							'Content-Type' : contentType
						});
						response.end(content, 'utf-8');
					});
				} else {
					response.writeHead(500);
					response.end('Sorry, check with the site admin for error: ' + error.code + ' ..\n');
					response.end();
				}
			} else {
				response.writeHead(200, {
					'Content-Type' : contentType
				});
				response.end(content, 'utf-8');
			}
		})
	}
}).listen(port);
console.log('Server running at http://127.0.0.1:8080');

function createJsonRespondWithComPorts(count, comPorts) {
	var resultString = "{\"count\":\"" + count + "\",\"comPorts\":[";
	var mask = 32768;
	for (var i = 0; i < count; i++) {
		resultString += "\"" + comPorts[i] + "\"";
		if (i != count - 1)
			resultString += ",";
	}
	resultString += "]}";
	return resultString;
}

function createJsonRespondWithStates(relayStates) {
	var resultString = "{\"relays\":[";
	var mask = 32768;
	for (var i = 0; i < 15; i++) {
		if (relayStates & mask)
			resultString += "\"1\",";
		else
			resultString += "\"0\",";
		mask /= 2;
	}
	if (relayStates & 1)
		resultString += "\"1\"]}";
	else
		resultString += "\"0\"]}";
	return resultString;
}